"""

USAGE:
    Format:
        format1
            clpaws azmon --check-az -a AVAILABILITY_ZONE

    Argument:
        --check-az           : check Availability Zone
        --debug              : output debug log (optional)
        -a AVAILABILITY_ZONE : Availability Zone
        -t TIMEOUT           : timeout (optional)

    ReturnCode:
        value of clpaws.errors.ERRORNO

    Example:
        example1
            clpaws azmon --check-az -a ap-northeast-1a
"""


import sys
import optparse
import clpaws.utils as util
import clpaws.errors as error
import logging

# global varivable
loglevel = logging.WARNING

def _check_option():
    parser = optparse.OptionParser()

    # required long option
    parser.add_option('--check-az',
                        dest='check_az',
                        action='store_true',
                        help='check Availability Zone')

    # optional long option
    parser.add_option('--debug',
                        dest='debug',
                        action='store_true',
                        help='output debug log')

    # required short option
    parser.add_option('-a',
                        dest='availability_zone',
                        help='Availability Zone',
                        metavar='AVAILABILITY_ZONE')

    # optional short option
    parser.add_option('-t',
                    dest='timeout',
                    help='timeout',
                    metavar='TIMEOUT')

    (options, args) = parser.parse_args()

    if not options.check_az:
        parser.error('argument "--check-az" is required.')

    if not options.availability_zone:
        parser.error('argument "-a" is required '
                     'when argument "--check-az" is specified.')

    return options


def check_availability_zone(availability_zone, ec2_cmdopt, timeout):
    logging.debug('Start check_availability_zone.')

    zones = _describe_availability_zones(availability_zone, ec2_cmdopt, timeout)
    if (zones and len(zones) == 1):
        state = zones[0]
        if state == 'available':
            pass
        else:
            err_msg = 'State of {0} is {1}.'.format(availability_zone, state)
            if state == 'information':
                raise error.AWSCLIError(error.ERRORNO['check_az_information'], err_msg)
            elif state == 'impaired':
                raise error.AWSCLIError(error.ERRORNO['check_az_impaired'], err_msg)
            elif state == 'unavailable':
                raise error.AWSCLIError(error.ERRORNO['check_az_unavailable'], err_msg)
            else:
                raise error.AWSCLIError(error.ERRORNO['check_az_error'], err_msg)
    else:
        err_msg = '{0} cannot be used.'.format(availability_zone)
        raise error.AWSCLIError(error.ERRORNO['check_az_error'], err_msg)

    logging.debug('End check_availability_zone.')


def _describe_availability_zones(availability_zone, ec2_cmdopt, timeout):
    if not ec2_cmdopt:
        cmdline = (util.CLPAWS_CMD + ' ec2 describe-availability-zones --zone-names "{0}" '
                   '--query \'AvailabilityZones[].State\''
                   .format(availability_zone))
    else:
        cmdline = (util.CLPAWS_CMD + ' ec2 describe-availability-zones --zone-names "{0}" '
                   '--query \'AvailabilityZones[].State\' {1}'
                   .format(availability_zone, ec2_cmdopt))
    result = util.execute_awscli(cmdline, timeout)
    return result


def main():
    global loglevel
    try:
        options = _check_option()

        if options.debug:
            loglevel = logging.DEBUG

        logging.basicConfig(level=loglevel,
                            format='%(levelname)s - %(message)s',
                            stream=sys.stdout)

        logging.debug('Start clpaws azmon.')

        util.set_environ()

        ec2_cmdopt = util.get_awscli_cmdopt('ec2')

        check_availability_zone(options.availability_zone, ec2_cmdopt, options.timeout)

        logging.debug('End clpaws azmon.')

        return error.ERRORNO['success']

    except error.AWSCLIError as ex:
        util.handle_exception(ex)
        return ex.value

    except Exception as ex:
        logging.error(ex)
        return error.ERRORNO['internal_error']


if __name__ == '__main__':
    sys.exit(main())
