"""

  clpaws dnsctl - control a resource record set in Amazon Route 53.

USAGE:
    Format:
        format1
            clpaws dnsctl --upsert-record -z HOSTED_ZONE_ID -r RECORD_NAME
                          -i RECORD_IP -s TTL -n RESOURCE_NAME
                          -w WAIT_INSYNC -x WAIT_NSYNC_TIMEOUT
        format2
            clpaws dnsctl --delete-record -z HOSTED_ZONE_ID -r RECORD_NAME
                          -i RECORD_IP -s TTL -n RESOURCE_NAME
        format3
            clpaws dnsctl --check-record -z HOSTED_ZONE_ID -r RECORD_NAME
                          -i RECORD_IP [-c]

    Argument:
        --upsert-record : add or change Amazon Route 53 resource record set
        --delete-record : delete Amazon Route 53 resource record set
        --check-record  : check to exist Route 53 resource record set
        --debug                 : output debug log (optional)
        -z HOSTED_ZONE_ID       : hosted zone id
        -r RECORD_NAME          : resource record set name
        -i RECORD_IP            : ip address of a resource record set
        -s TTL                  : ttl of a resource record set (sec)
        -n RESOURCE_NAME        : group resource name
        -c                      : enable check the resolving a domain name
        -w                      : wait for change request
        -x                      : wait for change request timeout (sec)
        -t TIMEOUT              : timeout (sec) (optional)

    ReturnCode:
        value of clpaws.errors.ERRORNO

"""


import json
import logging
import optparse
import os
import signal
import socket
import sys
import time

import clpaws.errors as error
import clpaws.utils as util

# constant variable
MODE_UPSERT = 'upsert'
MODE_DELETE = 'delete'

WAIT_INSYNC_INITIAL_TIME = 45
WAIT_INSYNC_INTERVAL_TIME = 5

# global variable
loglevel = logging.WARNING
json_file = ""


def _signal_handler(signum, stack):
    global json_file
    if not len(json_file) == 0:
        _delete_jsonfile(json_file)
    raise error.AWSCLIError(error.ERRORNO['internal_error'], "caught signal: " + str(signum))


def _check_option():
    exclusive_options = ('upsert_record', 'delete_record', 'check_record')

    parser = optparse.OptionParser()

    # exclusive long options
    parser.add_option('--upsert-record',
                      dest='upsert_record',
                      action='store_true',
                      help='change Amazon Route 53 resource record set')
    parser.add_option('--delete-record',
                      dest='delete_record',
                      action='store_true',
                      help='delete Amazon Route 53 resource record set')
    parser.add_option('--check-record',
                      dest='check_record',
                      action='store_true',
                      help='check Amazon Route 53 resource record set')

    # optional long option
    parser.add_option('--debug',
                      dest='debug',
                      action='store_true',
                      help='output debug log')

    # required short options
    parser.add_option('-z',
                      dest='hosted_zone_id',
                      help='hosted zone id',
                      metavar='HOSTED_ZONE_ID')
    parser.add_option('-r',
                      dest='record_name',
                      help='resource record set name',
                      metavar='RECORD_NAME')
    parser.add_option('-i',
                      dest='record_ip',
                      help='ip address of a resource record set',
                      metavar='RECORD_IP')
    parser.add_option('-s',
                      dest='record_ttl',
                      help='ttl of a resource record set',
                      metavar='RECORD_TTL')
    parser.add_option('-n',
                      dest='resource_name',
                      help='group resource name',
                      metavar='RESOURCE_NAME')
    parser.add_option('-c',
                      dest='check_resolve',
                      action='store_true',
                      help='enable check the resolving a domain name')
    parser.add_option('-w',
                      dest='wait_insync',
                      help='wait for change request',
                      metavar='WAIT_INSYNC')
    parser.add_option('-x',
                      dest='wait_insync_timeout',
                      help='wait for change request timeout',
                      metavar='WAIT_INSYNC_TIMEOUT')

    # optional short option
    parser.add_option('-t',
                      dest='timeout',
                      help='timeout',
                      metavar='TIMEOUT')

    (options, args) = parser.parse_args()

    toggled_options = [name for name in exclusive_options
                       if getattr(options, name)]

    if not len(toggled_options) == 1:
        parser.error('only one argument is required '
                     'among arguments "--upsert-record", "--delete-record", '
                     'and "--check-record".')

    if options.upsert_record and not (options.hosted_zone_id and
                                      options.record_name and
                                      options.record_ip and
                                      options.record_ttl and
                                      options.resource_name and
                                      options.wait_insync and
                                      options.wait_insync_timeout):
        parser.error('7 arguments "-z", "-r", "-i", "-s", "-n", "-w" and "-x" is required '
                     'when argument "--upsert-record" is specified.')

    if options.delete_record and not (options.hosted_zone_id and
                                      options.record_name and
                                      options.record_ip and
                                      options.record_ttl and
                                      options.resource_name):
        parser.error('5 arguments "-z", "-r", "-i", "-s" and "-n" is required '
                     'when argument "--delete-record" is specified.')

    if options.check_record and not (options.hosted_zone_id and
                                     options.record_name and
                                     options.record_ip):
        parser.error('3 arguments "-z", "-r" and "-i" is required '
                     'when argument "--check-record" is specified.')

    return options


def _wait_for_change_request(request_id, wait_insync_timeout, rt53_cmdopt, timeout=None):
    """Wait until reflected to the authoritative server in Amazon Route 53."""
    logging.debug('Start wait_for_change_request.')

    start_time = time.time()
    time.sleep(WAIT_INSYNC_INITIAL_TIME)
    if not rt53_cmdopt:
        cmdline = (util.CLPAWS_CMD + ' route53 get-change --id "{0}"'.format(request_id))
    else:
        cmdline = (util.CLPAWS_CMD + ' route53 get-change --id "{0}" {1}'.format(request_id, rt53_cmdopt))

    while True:
        result = util.execute_awscli(cmdline, timeout, err_msg='Failed to wait for change request.')
        if result['ChangeInfo']['Status'] == 'INSYNC':
            break

        elapsed_time = int(time.time() - start_time)
        if elapsed_time > int(wait_insync_timeout):
            logging.error('Wait for change request timed out.')
            break

        time.sleep(WAIT_INSYNC_INTERVAL_TIME)

    logging.debug('End wait_for_change_request.')


def upsert_record(hosted_zone_id, record_name, record_ip, record_ttl, resource_name, wait_insync, wait_insync_timeout, rt53_cmdopt, timeout=None):
    """Upsert a resource record set in Amazon Route 53."""
    global json_file
    logging.debug('Start upsert_record.')

    json_file = _make_json_path(MODE_UPSERT, resource_name)
    logging.debug('Json file name is "' + json_file + '".')

    _create_jsonfile(MODE_UPSERT, json_file, record_name, record_ip, record_ttl)

    try:
        if not rt53_cmdopt:
            cmdline = (util.CLPAWS_CMD + ' route53 change-resource-record-sets --output json '
                       '--hosted-zone-id "{0}" --change-batch "file://{1}"'
                       .format(hosted_zone_id, json_file))
        else:
            cmdline = (util.CLPAWS_CMD + ' route53 change-resource-record-sets {2} --output json '
                       '--hosted-zone-id "{0}" --change-batch "file://{1}"'
                       .format(hosted_zone_id, json_file, rt53_cmdopt))
        result = util.execute_awscli(cmdline, timeout, err_msg='Failed to change the resource record set \'{0}\' in Amazon Route 53.'.format(record_name))

        if int(wait_insync):
            _wait_for_change_request(result['ChangeInfo']['Id'], wait_insync_timeout, rt53_cmdopt, timeout)
    except KeyError:
        logging.error('KeyError has occurred. AWSCLI command succeeded, but no result was returned or the key did not exist.')
    finally:
        _delete_jsonfile(json_file)
        json_file = ""

    logging.debug('End upsert_record.')


def delete_record(hosted_zone_id, record_name, record_ip, record_ttl, resource_name, rt53_cmdopt, timeout=None):
    """Delete a resource record set in Amazon Route 53."""
    global json_file
    logging.debug('Start delete_record.')

    json_file = _make_json_path(MODE_DELETE, resource_name)
    logging.debug('Json file name is "' + json_file + '".')

    _create_jsonfile(MODE_DELETE, json_file, record_name, record_ip, record_ttl)

    try:
        if not rt53_cmdopt:
            cmdline = (util.CLPAWS_CMD + ' route53 change-resource-record-sets --output json '
                       '--hosted-zone-id "{0}" --change-batch "file://{1}"'
                       .format(hosted_zone_id, json_file))
        else:
            cmdline = (util.CLPAWS_CMD + ' route53 change-resource-record-sets {2} --output json '
                       '--hosted-zone-id "{0}" --change-batch "file://{1}"'
                       .format(hosted_zone_id, json_file, rt53_cmdopt))
        util.execute_awscli(cmdline, timeout, err_msg='Failed to delete the resource record set \'{0}\' in Amazon Route 53.'.format(record_name))
    finally:
        _delete_jsonfile(json_file)
        json_file = ""

    logging.debug('End delete_record.')


def check_record(hosted_zone_id, record_name, record_ip, rt53_cmdopt, check_resolve=False, timeout=None):
    """Monitor a resource record set in Amazon Route 53."""
    logging.debug('Start check_record.')

    records = _list_record(hosted_zone_id, record_name, rt53_cmdopt, timeout)
    if len(records) == 0:
        # resource record set not found.
        err_msg = 'The resource record set \'{0}\' in Amazon Route 53 does not exist.'.format(record_name)
        raise error.AWSCLIError(error.ERRORNO['check_route53_record_error'], err_msg)

    registered_ip = records[0]["ResourceRecords"][0]["Value"]
    if not (registered_ip == record_ip):
        # unexpected ip address of resource record set
        err_msg = 'Unexpected IP address of the resource record set \'{0}\' in Amazon Route 53. (registered: {1})'.format(record_name, registered_ip)
        raise error.AWSCLIError(error.ERRORNO['check_route53_record_ip_error'], err_msg)

    if check_resolve:
        try:
            resolved = socket.getaddrinfo(record_name, None)
        except Exception as e:
            # resolution failed
            err_msg = 'getaddrinfo(\'{0}\') failed: {1}'.format(record_name, e.args)
            raise error.AWSCLIError(error.ERRORNO['check_dns_resolve_error'], err_msg)

        resolved_ip = resolved[0][4][0]
        if not (resolved_ip == record_ip):
            # unexpected ip address of resolution domain name
            err_msg = 'Unexpected IP address which is resolved the record set name \'{0}\' from the DNS resolver. (resolved: {1})'.format(record_name, resolved_ip)
            raise error.AWSCLIError(error.ERRORNO['check_dns_resolved_ip_error'], err_msg)

    logging.debug('End check_record.')


def _list_record(hosted_zone_id, record_name, rt53_cmdopt, timeout):
    if not rt53_cmdopt:
        cmdline = (util.CLPAWS_CMD + ' route53 list-resource-record-sets --output json '
                   '--hosted-zone-id "{0}" --query "ResourceRecordSets[?Name == \'{1}\']"'
                   .format(hosted_zone_id, record_name))
    else:
        cmdline = (util.CLPAWS_CMD + ' route53 list-resource-record-sets {2} --output json '
                   '--hosted-zone-id "{0}" --query "ResourceRecordSets[?Name == \'{1}\']"'
                   .format(hosted_zone_id, record_name, rt53_cmdopt))
    result = util.execute_awscli(cmdline, timeout)
    return result


def _make_json_path(mode, resource_name):
    cur_dir = os.path.dirname(__file__)
    install_dir = os.path.join(cur_dir, os.pardir + os.sep + os.pardir + os.sep + os.pardir)
    work_dir = os.path.abspath(os.path.join(install_dir, 'work' + os.sep + 'awsdns'))

    if not os.path.isdir(work_dir):
        os.makedirs(work_dir)

    json_file = work_dir + os.sep + mode + '_record_' + resource_name + '.json'
    return json_file


def _create_jsonfile(mode, filename, record_name, record_ip, record_ttl):
    table = {
        MODE_UPSERT: {'comment': 'Modifying an A record', 'action': 'UPSERT'},
        MODE_DELETE: {'comment': 'Deleting an A record',  'action': 'DELETE'}
    }

    json_data = {
        'Comment': table[mode]['comment'],
        'Changes': [
            {
                'Action': table[mode]['action'],
                'ResourceRecordSet': {
                    'Name': record_name,
                    'Type': 'A',
                    'TTL': int(record_ttl),
                    'ResourceRecords': [
                        {'Value': record_ip}
                    ]}
            }
        ]}

    with open(filename, 'w') as openfile:
        openfile.write(json.dumps(json_data))


def _delete_jsonfile(filename):
    try:
        os.remove(filename)
    except Exception as ex:
        logging.warning(ex)


def main():
    """Main routine."""
    global loglevel
    try:
        options = _check_option()

        if options.debug:
            loglevel = logging.DEBUG

        logging.basicConfig(level=loglevel,
                            format='%(levelname)s - %(message)s',
                            stream=sys.stdout)

        logging.debug('Start clpaws dnsctl.')

        util.set_environ()

        rt53_cmdopt = util.get_awscli_cmdopt('route53')

        if not os.name == 'nt':  # Linux
            signal.signal(signal.SIGHUP, _signal_handler)
            signal.signal(signal.SIGINT, _signal_handler)
            signal.signal(signal.SIGQUIT, _signal_handler)
            signal.signal(signal.SIGTERM, _signal_handler)

        if options.upsert_record:
            upsert_record(options.hosted_zone_id,
                          options.record_name,
                          options.record_ip,
                          options.record_ttl,
                          options.resource_name,
                          options.wait_insync,
                          options.wait_insync_timeout,
                          rt53_cmdopt,
                          options.timeout)
        elif options.delete_record:
            delete_record(options.hosted_zone_id,
                          options.record_name,
                          options.record_ip,
                          options.record_ttl,
                          options.resource_name,
                          rt53_cmdopt,
                          options.timeout)
        elif options.check_record:
            check_record(options.hosted_zone_id,
                         options.record_name,
                         options.record_ip,
                         rt53_cmdopt,
                         options.check_resolve,
                         options.timeout)
        else:
            err_msg = 'Invalid option.'
            raise error.AWSCLIError(error.ERRORNO['internal_error'], err_msg)

        logging.debug('End clpaws dnsctl.')

        return error.ERRORNO['success']

    except error.AWSCLIError as ex:
        util.handle_exception(ex)
        return ex.value

    except Exception as ex:
        logging.error(ex)
        return error.ERRORNO['internal_error']

if __name__ == '__main__':
    sys.exit(main())
