"""

USAGE:
    Format:
        format1
            clpaws enichk --get-private-ip -n EC2_NETWORK_IF_ID -o FILE

    Argument:
        --get-private-ip     : get Primary private IP address
        --debug              : output debug log (optional)
        -n EC2_NETWORK_IF_ID : EC2 Network Interface ID
        -o FILE              : output file
        -t TIMEOUT           : timeout (optional)

    ReturnCode:
        value of clpaws.errors.ERRORNO

    Example:
        example1
            clpaws enichk --get-private-ip -n eni-00000000
                                           -o /tmp/workfile
"""


import sys
import optparse
import clpaws.utils as util
import clpaws.errors as error
import logging

# global varivable
loglevel = logging.WARNING

def _check_option():
    parser = optparse.OptionParser()

    # required long option
    parser.add_option('--get-private-ip',
                        dest='get_private_ip',
                        action='store_true',
                        help='get prmary private IP')

    # optional long option
    parser.add_option('--debug',
                        dest='debug',
                        action='store_true',
                        help='output debug log')

    # required short options
    parser.add_option('-n',
                        dest='ec2_network_if_id',
                        help='EC2 Network Interface ID',
                        metavar='EC2_NETWORK_INTERFACE_ID')
    parser.add_option('-o',
                        dest='filename',
                        help='output File',
                        metavar='FILE')

    # optional short option
    parser.add_option('-t',
                        dest='timeout',
                        help='timeout',
                        metavar='TIMEOUT')

    (options, args) = parser.parse_args()

    if not options.get_private_ip:
        parser.error('argument "--get-private-ip" is required.')

    if not (options.ec2_network_if_id and options.filename):
        parser.error('argument "-n" amd "-o" are required '
                        'when argument "--get-private-ip" is specified.')

    return options


def get_private_ip_address(ec2_network_if_id, filename, ec2_cmdopt, timeout):
    logging.debug('Start get_private_ip_address.')

    if not ec2_cmdopt:
        cmdline = (util.CLPAWS_CMD + ' ec2 describe-network-interfaces --network-interface-ids "{0}" '
                   '--query \'NetworkInterfaces[].PrivateIpAddresses[?Primary==`true`].PrivateIpAddress\''
                    .format(ec2_network_if_id))
    else:
        cmdline = (util.CLPAWS_CMD + ' ec2 describe-network-interfaces --network-interface-ids "{0}" '
                   '--query \'NetworkInterfaces[].PrivateIpAddresses[?Primary==`true`].PrivateIpAddress\' {1}'
                    .format(ec2_network_if_id, ec2_cmdopt))
    result = util.execute_awscli(cmdline, timeout)
    if (result and len(result) > 0 and len(result[0]) == 1):
        private_ip_address = result[0][0]
    else:
        err_msg = 'NetworkInterfaces is invalid.'
        raise error.AWSCLIError(error.ERRORNO['internal_error'], err_msg)

    with open(filename, 'w') as f:
        f.write(private_ip_address)

    logging.debug('End get_private_ip_address.')


def main():
    global loglevel
    try:
        options = _check_option()

        if options.debug:
            loglevel = logging.DEBUG

        logging.basicConfig(level=loglevel,
                            format='%(levelname)s - %(message)s',
                            stream=sys.stdout)

        logging.debug('Start clpaws enichk.')

        util.set_environ()

        ec2_cmdopt = util.get_awscli_cmdopt('ec2')

        get_private_ip_address(options.ec2_network_if_id, options.filename, ec2_cmdopt,
                                options.timeout)

        logging.debug('End clpaws enichk.')

        return error.ERRORNO['success']

    except error.AWSCLIError as ex:
        util.handle_exception(ex)
        return ex.value

    except Exception as ex:
        logging.error(ex)
        return error.ERRORNO['internal_error']


if __name__ == '__main__':
    sys.exit(main())
