#!/bin/bash
#
##
# clpawssip-watch.sh
##
# [coding：utf-8]
##


##
# function
##

# clpcloudutil calls
CURRENT=$(cd $(dirname $0);pwd)
. "${CURRENT}/../common/clpcloudutil.sh"


##
# monitoring the clpawssip
##

echo "clpawssip watch script start."


##
# ret code
##
AWSSIPW_SUCCESS_CODE=0					# succeed
AWSSIPW_ERR_SCHAWSCLI_CODE=50			# failed to search aws cli path
AWSSIPW_ERR_GETOCFENV_CODE=51			# failed to get the ocf environment variable
AWSSIPW_ERR_GETSECIP_CODE=52			# get secondary ip has failed
AWSSIPW_ERR_GETENIID_CODE=53			# get eni id has failed
AWSSIPW_ERR_AWSCHKIP_CODE=54			# failed to check ip address for aws
AWSSIPW_ERR_OSCHKIP_CODE=55				# failed to check ip address for os

AWSSIPW_WARN_AWSCHKIP_CODE=150			# failed to check ip address for aws


##
# resultmsg
##
AWSSIPW_ERR_SCHAWSCLI_MSG="The AWS CLI command is not found."
AWSSIPW_ERR_GETOCFENV_MSG="Failed to obtain the setting value."
AWSSIPW_ERR_GETSECIP_MSG="Failed to obtain the secondary IP address."
AWSSIPW_ERR_GETENIID_MSG="Failed to obtain the ENI ID."
AWSSIPW_ERR_AWSCHKIP_MSG="Failed to process checking the secondary IP address on the AWS side."
AWSSIPW_ERR_OSCHKIP_MSG="Failed to process checking the secondary IP address on the OS side."


##
# set the value to use for the shmrmset command
##
RESOURCE_NAME="${CLP_RESOURCENAME}"		# resource name
RESOURCE_TYPE="awssipw"						# resource type

if [ -z "${RESOURCE_NAME}" ]
then
	echo "failed to obtain the value required for the shmrmset command." 1>&2
fi


##
# awscli path
##
export PATH=$PATH:/usr/local/bin


##
# awscli path search
##
which aws > /dev/null 2>&1

if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_SCHAWSCLI_MSG}" > /dev/null 2>&1
	echo "failed to search aws cli path." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_SCHAWSCLI_CODE}
fi


##
# set environment variables for use with the aws cli
##
clpcloudutil_env_init "${clpcloudutil_false}" "${CURRENT}/clpaws_setting.conf"


##
# get aws cli command line options
##
EC2_CMDOPT="`clpcloudutil_awscli_cmdopt \"${clpcloudutil_false}\" ec2`"


##
# set ocf environment variable
##
TARGET_RSC="${CLP_OCF_PARAM1}"		# resource name
MON_MODE="${CLP_OCF_PARAM2}"			# action when aws cli command failed to receive response
PING_TIMEOUT="${CLP_OCF_PARAM3}"		# ping timeout

if [ -z "${TARGET_RSC}" ] || [ -z "${MON_MODE}" ] || [ -z "${PING_TIMEOUT}" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_GETOCFENV_MSG}" > /dev/null 2>&1
	echo "failed to get the ocf environment variable." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_GETOCFENV_CODE}
fi


##
# set server name
##
SRV_FULLNAME="`uname -n`"				# get hostname (use output file path)
SRV_NAME="`echo ${SRV_FULLNAME%%.*}`"	# delete domain name


##
# get target resource config
##
SECONDARY_IP="`clpcfget -g /root/resource/awssip@\"${TARGET_RSC}\"/parameters/ip -p awssip`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_GETSECIP_MSG}" > /dev/null 2>&1
	echo "${SECONDARY_IP}" | head -n 1 1>&2
	echo "get secondary ip has failed." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_GETSECIP_CODE}
fi

ENI_ID="`clpcfget -g /root/resource/awssip@\"${TARGET_RSC}\"/parameters/eniid -p awssip -s \"${SRV_NAME}\"`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_GETENIID_MSG}" > /dev/null 2>&1
	echo "${ENI_ID}" | head -n 1 1>&2
	echo "get eni id has failed." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_GETENIID_CODE}
fi

echo "get config has succeeded (secip=${SECONDARY_IP} eniid=${ENI_ID})."


##
# determine whether the secondary IP address is ipv4 or ipv6
##
if [ `echo "${SECONDARY_IP}" | grep :` ]
then
	SECONDARYIP_TYPE="IPv6"
	echo "secondary ip address is ipv6."
else
	SECONDARYIP_TYPE="IPv4"
	echo "secondary ip address is ipv4."
fi


##
# check secondary ip exists in aws
##
if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
then
	AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
							--filters \"Name=ipv6-addresses.ipv6-address,Values=\"${SECONDARY_IP}\"\" ${EC2_CMDOPT} --output text`"
elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
then
	AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
							--filters \"Name=addresses.private-ip-address,Values=\"${SECONDARY_IP}\"\" ${EC2_CMDOPT} --output text`"
fi

if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_AWSCHKIP_MSG}" > /dev/null 2>&1
	echo "failed to check the existence of ip address of aws." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	case ${MON_MODE} in
		0 ) exit ${AWSSIPW_SUCCESS_CODE} ;;
		1 ) exit ${AWSSIPW_WARN_AWSCHKIP_CODE} ;;
		2 ) exit ${AWSSIPW_ERR_AWSCHKIP_CODE} ;;
	esac
elif [ -z "${AWS_CHECK_IP}" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_AWSCHKIP_MSG}" > /dev/null 2>&1
	echo "failed to check the existence of ip address of aws." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_AWSCHKIP_CODE}
fi


##
# check secondary ip exists in os
##
clpping -c ${PING_TIMEOUT} "${SECONDARY_IP}" > /dev/null 2>&1
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIPW_ERR_OSCHKIP_MSG}" > /dev/null 2>&1
	echo "failed to check ip address for os." 1>&2
	echo "clpawssip watch script has failed." 1>&2
	exit ${AWSSIPW_ERR_OSCHKIP_CODE}
fi


##
# end
##
echo "clpawssip watch script has succeeded."

exit ${AWSSIPW_SUCCESS_CODE}

